#include "stdafx.h"
#include "clf_util.h"
#include "clf_random.h"
#include "clf_primitives.h"
#include "clf_math.h"
#include "clf_camera.h"
#include "clf_resources.h"
#include "clf_light.h"
#include "clf_font.h"
#include "clf_shader.h"
#include "clf_skybox.h"
#include "clf_effects.h"
#include "clf_quaternion.h"

#include "clf_world.h"
#include "clf_mesh.h"

extern HWND  hWnd;						// Window Handle
extern bool keyDown[256];			// holds true / false for each key

static int m_nFPS = 0;
static int m_nFrames = 0;											// FPS and FPS Counter
static bool g_pause=false;

cCamera g_cam;
clf_stage g_stage;

// a light source
clf_light g_light(GL_LIGHT0);

// the shader
clf_shader shader;

clf_skybox g_skybox(80);  // size of skybox is small enough for fog effect!!

// class to handle the fog effect
clf_fog g_fog;

// the game world
clf_world g_w;

clf_mesh g_Mesh;

void create_world()
{
// Mesh Generation Paramaters
#define MESH_RESOLUTION 6.f									// Pixels Per Vertex
#define MESH_LENGTHSCALE .66f									// Mesh Length scale
#define MESH_HEIGHTSCALE .2f									// Mesh Height scale
	// load Our Heightmap
	if( !g_Mesh.load_height_map( "terrain.tga", MESH_HEIGHTSCALE, MESH_LENGTHSCALE, MESH_RESOLUTION ) )
	{
		MessageBox( NULL, "Error Loading Heightmap", "Error", MB_OK );
		return ;
	}
}

void destroy_world()
{
	std::list<clf_world_object*>::iterator it;
	for (it=g_w.m_objects.begin(); it!=g_w.m_objects.end(); ++it)
		delete (*it);
}


///////////////////////////////////////////////////////////////////////
void restart_game()
{
}

// this function is called when the program starts to initialize the game
void game_init()
{
	init_math_lib();
	get_font_manager().create_font_2d("Bookman2D", "Bookman Old Style", 16);

	g_cam.setup(clf_vector3D(600,100,0),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up

	glEnable (GL_DEPTH_TEST);        // enable depth test
	glEnable(GL_COLOR_MATERIAL);     // use color for drawing the object

	// load shader
	shader.load_shaders("phong_vertex_shader2.glsl", "fragment_shader1.glsl");
	
	// Light model parameters:
	// -------------------------------------------
	// create and position a light source
	g_light.set_ambient(0.25f, 0.25f, 0.25f, 1.f);
	g_light.set_diffuse(0.7f, 0.7f, 0.7f, 1.f);
	g_light.set_specular(1.f, 1.f, 1.f, 1.f);
	g_light.set_position(-100.f, 100.f, 100.f, 0.f);

	// -------------------------------------------
	// Material parameters:
	
	GLfloat material_ambient[] = {0.7f, 0.7f, 0.7f, 1.0f};
	GLfloat material_diffuse[] = {0.4f, 0.4f, 0.4f, 1.0f};
	GLfloat material_specular[] = {0.8f, 0.8f, 0.8f, 1.0f};
	GLfloat material_emission[] = {0.1f, 0.1f, 0.1f, 0.0f};
	GLfloat material_shine = 50.0f;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, material_emission);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, material_shine);

	g_skybox.load_textures("skybox/up.tga", "skybox/dn.tga", "skybox/ft.tga", "skybox/bk.tga", "skybox/lt.tga", "skybox/rt.tga");

	create_world();

	restart_game();
}

// this function is called when the window is resized
void resize_window (int width, int height)			// reshape the window when it's moved or resized
{
	// setup viewport
	g_stage.resize(0,0,width, height);
}

// this function is called upon exit to clear allocated resources
void game_terminate()
{
	destroy_world();
	shader.terminate();
}

// the core of the game. This is called repeatedly to update the state of the game
// fElapsed: is the elapsed time since last call. The time is in seconds (usualy a very small number)
void frame_move(float fElapsed)
{
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);
	if (keyDown['R'])
	{
		keyDown['R'] = 0;
		restart_game();
	}
	if (keyDown['P'])
	{
		keyDown['P'] = 0;
		g_pause = !g_pause;
	}

	if (!g_pause)
	{
		g_w.step_simulation(fElapsed);
	}

	// frame counting mechanism
	// the higher the frame rate the faster the system
	// if frame rate drops below 30 we are in deep trouble
	// we should either optimize the program or buy a new computer
	static dFloat tot = 0;
	tot += fElapsed;
	m_nFrames++;												// Increment Our frame Counter
	if (tot >= 1.f)
	{
		tot = 0;
		m_nFPS = m_nFrames;
		m_nFrames = 0;

		char txt[200];
		sprintf(txt, "GusOnGames, fps:%d",m_nFPS);
		SetWindowText(hWnd, txt);
	}
}

void frame_render()
{
	// clear the screen
	// drawing the skybox means we do not have to waste time clearing the screen
	glClear(/*GL_COLOR_BUFFER_BIT |*/ GL_DEPTH_BUFFER_BIT /* | GL_STENCIL_BUFFER_BIT */);	// clear screen and stencil buffer

	glMatrixMode (GL_PROJECTION);									// select the projection matrix
	glLoadIdentity ();														// reset the projection matrix

	// viewport and perspective
	g_stage.prepare();

	glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
	glLoadIdentity ();													// reset

	glPushMatrix();

	// position the viewer
	g_cam.apply();

	glPushAttrib(GL_ALL_ATTRIB_BITS);

	glEnable(GL_NORMALIZE);
	// position the light
	g_light.enable();

	// fog
	g_fog.fog_apply();
	// render the skybox
	g_skybox.render(g_cam.vLocation.x,g_cam.vLocation.y,g_cam.vLocation.z);

	// shader
	shader.begin();

	// reapply the light
	g_light.apply();

	// draw world objects
	g_w.render();
	g_Mesh.render();

	shader.end();
	glPopMatrix();
	glPopAttrib();
}
